@@ -636,10 +636,10 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 	struct stats_net_dev *sndc, *sndp;
 	unsigned int index = 0;
 
-	sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + pos * a->msize);
+	sndc = (struct stats_net_dev *) a->buf[curr] + pos;
 
 	while (index < a->nr) {
-		sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
+		sndp = (struct stats_net_dev *) a->buf[ref] + index;
 		if (!strcmp(sndc->interface, sndp->interface)) {
 			/*
 			 * Network interface found.
@@ -709,16 +709,19 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 
 	/* Network interface not found: Look for the first free structure */
 	for (index = 0; index < a->nr; index++) {
-		sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
-		if (!strcmp(sndp->interface, ""))
+		sndp = (struct stats_net_dev *) a->buf[ref] + index;
+		if (!strcmp(sndp->interface, "?")) {
+			memset(sndp, 0, STATS_NET_DEV_SIZE);
+			strcpy(sndp->interface, sndc->interface);
 			break;
+		}
 	}
 	if (index >= a->nr) {
 		/* No free structure: Default is structure of same rank */
 		index = pos;
 	}
 
-	sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
+	sndp = (struct stats_net_dev *) a->buf[ref] + index;
 	/* Since the name is not the same, reset all the structure */
 	memset(sndp, 0, STATS_NET_DEV_SIZE);
 	strcpy(sndp->interface, sndc->interface);
@@ -748,10 +751,10 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 	struct stats_net_edev *snedc, *snedp;
 	unsigned int index = 0;
 
-	snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + pos * a->msize);
+	snedc = (struct stats_net_edev *) a->buf[curr] + pos;
 
 	while (index < a->nr) {
-		snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
+		snedp = (struct stats_net_edev *) a->buf[ref] + index;
 		if (!strcmp(snedc->interface, snedp->interface)) {
 			/*
 			 * Network interface found.
@@ -781,16 +784,19 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 
 	/* Network interface not found: Look for the first free structure */
 	for (index = 0; index < a->nr; index++) {
-		snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
-		if (!strcmp(snedp->interface, ""))
+		snedp = (struct stats_net_edev *) a->buf[ref] + index;
+		if (!strcmp(snedp->interface, "?")) {
+			memset(snedp, 0, STATS_NET_EDEV_SIZE);
+			strcpy(snedp->interface, snedc->interface);
 			break;
+		}
 	}
 	if (index >= a->nr) {
 		/* No free structure: Default is structure of same rank */
 		index = pos;
 	}
 
-	snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
+	snedp = (struct stats_net_edev *) a->buf[ref] + index;
 	/* Since the name is not the same, reset all the structure */
 	memset(snedp, 0, STATS_NET_EDEV_SIZE);
 	strcpy(snedp->interface, snedc->interface);
@@ -818,10 +824,10 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 	struct stats_disk *sdc, *sdp;
 	int index = 0;
 
-	sdc = (struct stats_disk *) ((char *) a->buf[curr] + pos * a->msize);
+	sdc = (struct stats_disk *) a->buf[curr] + pos;
 
 	while (index < a->nr) {
-		sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
+		sdp = (struct stats_disk *) a->buf[ref] + index;
 		if ((sdc->major == sdp->major) &&
 		    (sdc->minor == sdp->minor)) {
 			/*
@@ -846,16 +852,20 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 
 	/* Disk not found: Look for the first free structure */
 	for (index = 0; index < a->nr; index++) {
-		sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
-		if (!(sdp->major + sdp->minor))
+		sdp = (struct stats_disk *) a->buf[ref] + index;
+		if (!(sdp->major + sdp->minor)) {
+			memset(sdp, 0, STATS_DISK_SIZE);
+			sdp->major = sdc->major;
+			sdp->minor = sdc->minor;
 			break;
+		}
 	}
 	if (index >= a->nr) {
 		/* No free structure found: Default is structure of same rank */
 		index = pos;
 	}
 
-	sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
+	sdp = (struct stats_disk *) a->buf[ref] + index;
 	/* Since the device is not the same, reset all the structure */
 	memset(sdp, 0, STATS_DISK_SIZE);
 	sdp->major = sdc->major;
@@ -2102,3 +2112,232 @@ void replace_nonprintable_char(int ifd, char *comment)
 			comment[i] = '.';
 	}
 }
+
+/*
+ ***************************************************************************
+ * Fill the rectime and loctime structures with current record's date and
+ * time, based on current record's "number of seconds since the epoch" saved
+ * in file.
+ * For loctime (if given): The timestamp is expressed in local time.
+ * For rectime: The timestamp is expressed in UTC, in local time, or in the
+ * time of the file's creator depending on options entered by the user on the
+ * command line.
+ *
+ * IN:
+ * @l_flags	Flags indicating the type of time expected by the user.
+ * 		S_F_LOCAL_TIME means time should be expressed in local time.
+ * 		S_F_TRUE_TIME means time should be expressed in time of
+ * 		file's creator.
+ * 		Default is time expressed in UTC (except for sar, where it
+ * 		is local time).
+ * @record_hdr	Record header containing the number of seconds since the
+ * 		epoch, and the HH:MM:SS of the file's creator.
+ *
+ * OUT:
+ * @rectime	Structure where timestamp for current record has been saved
+ * 		(in local time or in UTC depending on options used).
+ * @loctime	If given, structure where timestamp for current record has
+ * 		been saved (expressed in local time). This field will be used
+ * 		for time comparison if options -s and/or -e have been used.
+ *
+ * RETURNS:
+ * 1 if an error was detected, or 0 otherwise.
+ ***************************************************************************
+*/
+int sa_get_record_timestamp_struct(unsigned int l_flags, struct record_header *record_hdr,
+				   struct tm *rectime, struct tm *loctime)
+{
+	struct tm *ltm = NULL;
+	int rc = 0;
+
+	/* Fill localtime structure if given */
+	if (loctime) {
+		if ((ltm = localtime((const time_t *) &(record_hdr->ust_time))) != NULL) {
+			*loctime = *ltm;
+		}
+		else {
+			rc = 1;
+		}
+	}
+
+	/* Fill generic rectime structure */
+	if (PRINT_LOCAL_TIME(l_flags) && !ltm) {
+		/* Get local time if not already done */
+		ltm = localtime((const time_t *) &(record_hdr->ust_time));
+	}
+
+	if (!PRINT_LOCAL_TIME(l_flags) && !PRINT_TRUE_TIME(l_flags)) {
+		/*
+		 * Get time in UTC
+		 * (the user doesn't want local time nor time of file's creator).
+		 */
+		ltm = gmtime((const time_t *) &(record_hdr->ust_time));
+	}
+
+	if (ltm) {
+		/* Done even in true time mode so that we have some default values */
+		*rectime = *ltm;
+	}
+	else {
+		rc = 1;
+	}
+
+	if (PRINT_TRUE_TIME(l_flags)) {
+		/* Time of file's creator */
+		rectime->tm_hour = record_hdr->hour;
+		rectime->tm_min  = record_hdr->minute;
+		rectime->tm_sec  = record_hdr->second;
+	}
+
+	return rc;
+}
+
+/*
+ ***************************************************************************
+ * Set current record's timestamp strings (date and time) using the time
+ * data saved in @rectime structure. The string may be the number of seconds
+ * since the epoch if flag S_F_SEC_EPOCH has been set.
+ *
+ * IN:
+ * @l_flags	Flags indicating the type of time expected by the user.
+ * 		S_F_SEC_EPOCH means the time should be expressed in seconds
+ * 		since the epoch (01/01/1970).
+ * @record_hdr	Record header containing the number of seconds since the
+ * 		epoch.
+ * @cur_date	String where timestamp's date will be saved. May be NULL.
+ * @cur_time	String where timestamp's time will be saved.
+ * @len		Maximum length of timestamp strings.
+ * @rectime	Structure with current timestamp (expressed in local time or
+ *		in UTC depending on whether options -T or -t have been used
+ * 		or not) that should be broken down in date and time strings.
+ *
+ * OUT:
+ * @cur_date	Timestamp's date string (if expected).
+ * @cur_time	Timestamp's time string. May contain the number of seconds
+ *		since the epoch (01-01-1970) if corresponding option has
+ * 		been used.
+ ***************************************************************************
+*/
+void set_record_timestamp_string(unsigned int l_flags, struct record_header *record_hdr,
+				 char *cur_date, char *cur_time, int len, struct tm *rectime)
+{
+	/* Set cur_time date value */
+	if (PRINT_SEC_EPOCH(l_flags) && cur_date) {
+		sprintf(cur_time, "%ld", record_hdr->ust_time);
+		strcpy(cur_date, "");
+	}
+	else {
+		/*
+		 * If options -T or -t have been used then cur_time is
+		 * expressed in local time. Else it is expressed in UTC.
+		 */
+		if (cur_date) {
+			strftime(cur_date, len, "%Y-%m-%d", rectime);
+		}
+		if (USE_PREFD_TIME_OUTPUT(l_flags)) {
+			strftime(cur_time, len, "%X", rectime);
+		}
+		else {
+			strftime(cur_time, len, "%H:%M:%S", rectime);
+		}
+	}
+}
+
+/*
+ ***************************************************************************
+ * Print contents of a special (RESTART or COMMENT) record.
+ *
+ * IN:
+ * @record_hdr	Current record header.
+ * @l_flags	Flags for common options.
+ * @tm_start	Structure filled when option -s has been used.
+ * @tm_end	Structure filled when option -e has been used.
+ * @rtype	Record type (R_RESTART or R_COMMENT).
+ * @ifd		Input file descriptor.
+ * @rectime	Structure where timestamp (expressed in local time or in UTC
+ *		depending on whether options -T/-t have	been used or not) can
+ *		be saved for current record.
+ * @loctime	Structure where timestamp (expressed in local time) can be
+ *		saved for current record. May be NULL.
+ * @file	Name of file being read.
+ * @tab		Number of tabulations to print.
+ * @file_magic	file_magic structure filled with file magic header data.
+ * @file_hdr	System activity file standard header.
+ * @act		Array of activities.
+ * @ofmt		Pointer on report output format structure.
+ *
+ * OUT:
+ * @rectime		Structure where timestamp (expressed in local time
+ * 			or in UTC) has been saved.
+ * @loctime		Structure where timestamp (expressed in local time)
+ * 			has been saved (if requested).
+ *
+ * RETURNS:
+ * 1 if the record has been successfully displayed, and 0 otherwise.
+ ***************************************************************************
+ */
+int print_special_record(struct record_header *record_hdr, unsigned int l_flags,
+			 struct tstamp *tm_start, struct tstamp *tm_end, int rtype, int ifd,
+			 struct tm *rectime, struct tm *loctime, char *file, int tab,
+			 struct file_magic *file_magic, struct file_header *file_hdr,
+			 struct activity *act[], struct report_format *ofmt)
+{
+	char cur_date[32], cur_time[32];
+	int dp = 1;
+	unsigned int new_cpu_nr;
+
+	/* Fill timestamp structure (rectime) for current record */
+	if (sa_get_record_timestamp_struct(l_flags, record_hdr, rectime, loctime))
+		return 0;
+
+	/* If loctime is NULL, then use rectime for comparison */
+	if (!loctime) {
+		loctime = rectime;
+	}
+
+	/* The record must be in the interval specified by -s/-e options */
+	if ((tm_start->use && (datecmp(loctime, tm_start) < 0)) ||
+	    (tm_end->use && (datecmp(loctime, tm_end) > 0))) {
+		/* Will not display the special record */
+		dp = 0;
+	}
+	else {
+		/* Set date and time strings to be displayed for current record */
+		set_record_timestamp_string(l_flags, record_hdr,
+					    cur_date, cur_time, 32, rectime);
+	}
+
+	if (rtype == R_RESTART) {
+		/* Don't forget to read the volatile activities structures */
+		new_cpu_nr = read_vol_act_structures(ifd, act, file, file_magic,
+						     file_hdr->sa_vol_act_nr);
+
+		if (!dp)
+			return 0;
+
+		if (*ofmt->f_restart) {
+			(*ofmt->f_restart)(&tab, F_MAIN, cur_date, cur_time,
+					   !PRINT_LOCAL_TIME(l_flags) &&
+					   !PRINT_TRUE_TIME(l_flags), file_hdr,
+					   new_cpu_nr);
+		}
+	}
+	else if (rtype == R_COMMENT) {
+		char file_comment[MAX_COMMENT_LEN];
+
+		/* Read and replace non printable chars in comment */
+		replace_nonprintable_char(ifd, file_comment);
+
+		if (!dp || !DISPLAY_COMMENT(l_flags))
+			return 0;
+
+		if (*ofmt->f_comment) {
+			(*ofmt->f_comment)(&tab, F_MAIN, cur_date, cur_time,
+					   !PRINT_LOCAL_TIME(l_flags) &&
+					   !PRINT_TRUE_TIME(l_flags), file_comment,
+					   file_hdr);
+		}
+	}
+
+	return 1;
+}
